﻿using System;
using System.Collections.Generic;
using System.Text;
using SuperMap.Data;
using SuperMap.UI;
using SuperMap.Realspace;

namespace SuperMap.Desktop.Samples
{
    class _Toolkit
    {
        public enum Direction
        {
            Left,  
            Right, 
            Up,  
            Down, 
        }
        /// <summary>
        /// On the user perspective, move along a direction
        /// </summary>
        /// <param name="direction">Direction</param>
        /// <param name="distance">Distance (m)</param>
        public static void changeGeometryPosition(Direction direction,Double distance)
        {
            Recordset recordset = null;
            Geometry geometry = null;
            try
            {
                IFormScene formScene = SuperMap.Desktop.Application.ActiveForm as IFormScene;
                SceneControl sceneControl = formScene.SceneControl;
                Selection3D[] selection3Ds = sceneControl.Scene.FindSelection(true);
                if (selection3Ds.Length ==1)
                {
                    Selection3D selection = selection3Ds[0];
                    recordset = selection.ToRecordset();
                    if (selection.Layer.Type == Layer3DType.Dataset && selection.Count == 1)
                    {
                        Int32 selectedId = (int)recordset.GetFieldValue("SmID");
                        geometry = recordset.GetGeometry();
                        
                        Point2D souPoint2D = geometry.InnerPoint;
                        Point2D tarPoint2D = _Toolkit.caulatePosition(souPoint2D, direction, distance);
                        
                        //Update position of the selected object by the new point
                        geometry.Offset(tarPoint2D.X - geometry.InnerPoint.X, tarPoint2D.Y - geometry.InnerPoint.Y);
                        recordset.Edit();
                        recordset.SetGeometry(geometry);
                        recordset.Update();
                        selection.Layer.UpdateData();
                        selection.Add(selectedId);
                        selection.UpdateData();
                    }
                }
            }
            catch (Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            finally
            {
                if (geometry != null) { geometry.Dispose(); }
                if (recordset != null)
                {
                    recordset.Close();
                    recordset.Dispose();
                }
            }
        }

        /// <summary>
        /// Calculate the position of the current selected object after moving
        /// </summary>
        /// <param name="souPoint2D">The position of selected object on the scene</param>
        /// <param name="direction">Moving Direction (up, down, left and right)</param>
        /// <param name="distance">Moving Distance (m)</param>
        /// <returns></returns>
        public static Point2D caulatePosition(Point2D souPoint2D,Direction direction,Double distance)
        {
            Point2D result = Point2D.Empty;
            try
            {
                IFormScene formScene = SuperMap.Desktop.Application.ActiveForm as IFormScene;
                SceneControl sceneControl = formScene.SceneControl;
                Double heading = sceneControl.Scene.Camera.Heading / 180 * 3.1415926; //Convert angle into radian

                //Convert the geographic coordinate system into projected coordinate system
                Point2Ds souPoint2Ds = new Point2Ds(new Point2D[] { souPoint2D });
                PrjCoordSys dePrjCoordSys = new PrjCoordSys(PrjCoordSysType.Wgs1984Utm50N); //目标投影坐标系设置为Wgs1984Utm50N
                CoordSysTranslator.Forward(souPoint2Ds, dePrjCoordSys);

                //Calculate the position of end point
                Point2D tarPoint2D = new Point2D();
                switch (direction)
                {
                    case Direction.Left:
                        tarPoint2D = new Point2D(souPoint2Ds[0].X - distance * Math.Cos(heading), souPoint2Ds[0].Y + distance * Math.Sin(heading));
                        break;
                    case Direction.Right:
                        tarPoint2D = new Point2D(souPoint2Ds[0].X + distance * Math.Cos(heading), souPoint2Ds[0].Y - distance * Math.Sin(heading));
                        break;
                    case Direction.Up:
                        tarPoint2D = new Point2D(souPoint2Ds[0].X + distance * Math.Sin(heading), souPoint2Ds[0].Y + distance * Math.Cos(heading));
                        break;
                    case Direction.Down:
                        tarPoint2D = new Point2D(souPoint2Ds[0].X - distance * Math.Sin(heading), souPoint2Ds[0].Y - distance * Math.Cos(heading));
                        break;
                    default:
                        break;
                }

                //Convert the coordinate sytem of the moved object into geographic coordinate system
                Point2Ds tarPoint2Ds = new Point2Ds();
                tarPoint2Ds.Add(tarPoint2D);
                CoordSysTranslator.Inverse(tarPoint2Ds, dePrjCoordSys);
                result = tarPoint2Ds[0];
            }
            catch (Exception ex)
            {
            	Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }

            return result;
        }

        /// <summary>
        /// Change the streching height of layer items of Unique or Range Thematic map, and change the streching height of common vector layers
        /// </summary>
        /// <param name="height">Greater than or equal to 0 means increasing height; Less than 0 means decreasing height</param>
        public static void changeHeight(Double height)
        {
            Recordset recordset = null;
            Geometry geometry = null;
            try
            {
                IFormScene formScene = SuperMap.Desktop.Application.ActiveForm as IFormScene;
                SceneControl sceneControl = formScene.SceneControl;
                Selection3D[] selection3Ds = sceneControl.Scene.FindSelection(true);
                if (selection3Ds.Length == 1 && selection3Ds[0].Layer.Type == Layer3DType.Dataset && (selection3Ds[0].Layer as Layer3DDataset).Dataset.Type==DatasetType.Region)
                {
                    Selection3D selection = selection3Ds[0];
                    Int32 selectedId = selection[0];
                    Theme3D theme3D = (selection.Layer as Layer3DDataset).Theme;
                    String action = (height >= 0) ? "Increase" : "Decrease";
                    String message = "";

                    
                    if (theme3D is Theme3DUnique)
                    {
                        recordset = selection.ToRecordset();
                        Theme3DUnique theme3DUnique = theme3D as Theme3DUnique;
                        String itemString = recordset.GetFieldValue(theme3DUnique.UniqueExpression).ToString();
                        Theme3DUniqueItem uniqueItem = theme3DUnique[theme3DUnique.IndexOf(itemString)];
                        if (uniqueItem.IsModellingStyleEnabled)
                        {
                            uniqueItem.Style.ExtendedHeight = uniqueItem.Style.ExtendedHeight + height;
                            message = String.Format("The items {1} in Unique Values layer is {2}{3} meters.", selection.Layer.Caption,itemString, action, Math.Abs(height));
                        }
                    }
                    else if (theme3D is Theme3DRange)
                    {
                        recordset = selection.ToRecordset();
                        Theme3DRange theme3DRange = theme3D as Theme3DRange;
                        Double itemValue = (Double)recordset.GetFieldValue(theme3DRange.RangeExpression);
                        Theme3DRangeItem rangeItem = theme3DRange[theme3DRange.IndexOf(itemValue)];
                        if (rangeItem.IsModellingStyleEnabled)
                        {
                            rangeItem.Style.ExtendedHeight = rangeItem.Style.ExtendedHeight + height;
                            message = String.Format("The items {1} in Range layer is {2}{3} meters.", selection.Layer.Caption, rangeItem.Caption, action, Math.Abs(height));
                        }
                    }

                  
                    if (theme3D==null||message=="")
                    {
                        Layer3DSettingVector layer3DSettingVector = (selection.Layer as Layer3DDataset).AdditionalSetting as Layer3DSettingVector;
                        if (layer3DSettingVector.ExtendedHeightField != string.Empty)  //If ExtendedHeight is field, change the field value of the selected object
                        {
                            recordset = selection.ToRecordset();
                            geometry = recordset.GetGeometry();
                            Object preValue = recordset.GetFieldValue(layer3DSettingVector.ExtendedHeightField);
                            Object desValue = Convert.ToDouble(preValue) + height;
                            recordset.Edit();
                            recordset.SetFieldValue(layer3DSettingVector.ExtendedHeightField, desValue);
                            recordset.Update();
                            message = String.Format("Object “{0}” {1}{2} meters.", selectedId, action, Math.Abs(height));
                        }
                        else  //If ExtendedHeight is value, change the value directly
                        {
                            layer3DSettingVector.Style.ExtendedHeight = layer3DSettingVector.Style.ExtendedHeight + height;
                            message = String.Format("Layer “{0}” {1}{2} meters.", selection.Layer.Caption, action, Math.Abs(height));
                        }
                    }

                    selection.Layer.UpdateData();
                    selection.Add(selectedId);
                    selection.UpdateData();
                    if (message!="")
                    {
                        SuperMap.Desktop.Application.ActiveApplication.Output.Output(message);
                    }
                }
            }
            catch (Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            finally
            {
                if (geometry != null) { geometry.Dispose(); }
                if (recordset != null)
                {
                    recordset.Close();
                    recordset.Dispose();
                }
            }         
        }
    }
}
